#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include "portbits.h"
#include "param.h"
#include "fonds/4x6_horizontal_MSB_1.h"
#include "fonds/8x8_horizontal_MSB_1.h"
#include "fonds/8x12_horizontal_MSB_1.h"

#if flashsize >= 32
#include "fonds/12x16_horizontal_MSB_1.h"
#include "fonds/16x26_horizontal_MSB_1.h"
#endif
#if flashsize >= 64
#include "fonds/24x40_horizontal_MSB_1.h"
#endif

#include "global.h"
#include "lcd.h"

//unsigned char cgram[16][8];

extern unsigned char viewstart, viewstart2, Graymode, GrayCnt, charcol;
extern uint8_t textb_x;

register unsigned char R_RAS asm("r3");
register unsigned char R_CAS asm("r4");
register unsigned char R_RAS_W asm("r5");
register unsigned char R_CAS_W asm("r6");

void lcd_init(void)
{	
	cli();
	wdt_disable();

	R_RAS = C_RAS;
	R_CAS = C_CAS;
	R_RAS_W = C_RAS_W;
	R_CAS_W = C_CAS_W;


	lcd_clear();
#ifdef STARTMON
	lcd_string(6,10,PSTR("320x240 LCD Controller mit D-RAM"),3);
	lcd_string(6,24,PSTR("SW/4 GW Mode"),3);
	lcd_string(6,36,PSTR("\xB8"" by NRG"),3);
	lcd_string(6,48,PSTR("Bis zu 6 Fonts + 1 Benutzer Font"),3);
	
	lcd_string(6,60,PSTR("4x6 Font"),1);
	lcd_string(6,68,PSTR("8x8 Font"),2);
	lcd_string(6,78,PSTR("8x12 Font"),3);
#if flashsize >=32	
	lcd_string(6,92,PSTR("12x16 Font"),4);
	lcd_string(6,110,PSTR("16x26 Font"),5);
#if flashsize >=64
	lcd_string(6,138,PSTR("24x40 Font"),6);
#endif	
	charcol=0;
	uint8_t x=6;
	for(uint8_t i=0;i<4;i++){
		for(uint8_t j=0;j<4;j++){
			lcd_writechar(x,213,'X',5);
			x=x+4;
			charcol=charcol+4;
		}
		charcol=charcol&0b11000000;
		charcol=charcol+64;
	}
	charcol=0b00001111;
#endif
	
#endif

// Timer initialisierung fr Mega 164/324/644
#ifdef megaXX4
// Timer 2 fr LCD Zeilen Interrupt
	OCR2A=RELOAD;
	TCCR2A=(1<<WGM21);							// 2MHz Timer Takt, CTC
	TCCR2B=(1<<CS21);
	TIMSK2=(1<<OCIE2A);	
	
// LED / Kontrast PWM

	TCCR0A = (1<<COM0A1) | (0<<COM0A0) | (1<<COM0B1) | (0<<COM0B0) | (1<<WGM01) | (1<<WGM00) ;
	TCCR0B = (0<<FOC0A) | (0<<FOC0B) | (0<<WGM02) | (1<<CS02) | (0<<CS01) | (0<<CS00);
	OCR0A = 127;
	OCR0B = 127;
	TIMSK0 =0;
		
#ifdef PWMLOOP
// Timer1 fr Hardware LCD ausgabe
	TCCR1A = 0;
	TCCR1B = 0;
	TCCR1C =0;
	OCR1A = 1;
	OCR1B = 0;
	ICR1 = 2;
	TIMSK1 =0;
#endif
	
// Timer initialisierung fr Mega Mega 16/32
#else
// Timer 2 fr LCD Zeilen Interrupt	
	OCR2=RELOAD;
	TCCR2=(1<<WGM21)|(1<<CS21);
	TIMSK|=(1<<OCIE2);
	
// LED / Kontrast PWM      
	TCCR0 = (0<<FOC0) | (1<<WGM00) | (1<<COM01) | (0<<COM00) | (1<<WGM01) | (1<<CS02) | (0<<CS01) | (0<<CS00);
	OCR0=127;
	
#ifdef PWMLOOP
// Timer1 fr Hardware LCD ausgabe
	TCCR1A = 0;
	TCCR1B = 0;
	OCR1A = 1;
	OCR1B = 0;
	ICR1 = 2;
#endif	
	
#endif	
	sei();

	wdt_enable(WDTO_250MS);
	ENABLE_VLCD=1;
}
#ifdef STARTMON
void lcd_string(unsigned char x, unsigned char y,const char *txt, unsigned char font)
{	
	unsigned char c,temp=0;
	if(font == 1)
		temp=1;	
	else if(font <= 3)
		temp=2;
	else if(font == 4)  
		temp=3;
	else if (font == 5)
		temp=4;
#if flashsize >=64
	else if (font == 6)
		temp = 6;
#endif
	else if (font == 0)
		temp = textb_x;
			
	for(;;)
	{	c=pgm_read_byte(txt++);
		if (!c)
			break;
		lcd_writechar(x,y,c,font);
		x=x+temp;
	}
}
#endif
void bresenham_line (unsigned short x1, unsigned short y1, unsigned short x2, unsigned short y2, unsigned char color)
{	
	signed short dx,dy,x,y;
	signed char sdx,sdy;
	unsigned short i,px,py;
	dx=x2-x1;      // the horizontal distance of the line
	dy=y2-y1;      // the vertical distance of the line
	if (dx==0)
		sdx=0;
	if (dx>0)
		sdx=1;
	else
	{  	sdx=-1;
		dx=-dx;
	}
	if (dy==0)
		sdy=0;
	if (dy>0)
		sdy=1;
	else
	{	sdy=-1;
		dy=-dy;
	}
	x=dx*2;
	y=dy*2;
	px=x1;
	py=y1;
	lcd_setpixel (px,py,color);

	if (dx>=dy)					 // the line is more horizontal than vertical
	{	for(i=0;i<dx;i++)
		{	y+=dy;
			if (y>=dx)
			{	y-=dx;
				py+=sdy;
			}
			px+=sdx;
			lcd_setpixel (px,py,color);
		}
	}
	else				 		// the line is more vertical than horizontal
	{	for(i=0;i<dy;i++)
		{	x+=dx;
			if (x>=dy)
			{	x-=dy;
				px+=sdx;
			}
			py+=sdy;
			lcd_setpixel (px,py,color);
		}
	}
	
}

void lcd_line (unsigned short x1, unsigned short y1, unsigned short x2, unsigned short y2, unsigned char color)
{   
	unsigned short temp;
	if (y1==y2)
	{	if (x1>x2)
		{	temp=x1;
			x1=x2;
			x2=temp;
		}
		for (temp=x1; temp<=x2; temp++){
			if ((temp & 7)==0 && (x2-temp)>7){

				if(Graymode != 0)
					lcd_writebyteex(temp/8,y1,255,color,0);
				else{
					if(color & 0b10000000)
						lcd_writebyte(temp/8,y1,255);
					else
						lcd_writebyte(temp/8,y1,0);
				}								
				temp += 7;
			}
			else
				lcd_setpixel (temp,y1,color);
		}
	}
	else if (x1==x2)
	{	if (y1>y2)
		{	temp=y1;
			y1=y2;
			y2=temp;
		}
		for (temp=y1; temp<=y2; temp++)
			lcd_setpixel (x1,temp,color);
	}
	else
		bresenham_line (x1, y1, x2, y2, color);
}

void lcd_circle(unsigned short xm, unsigned short ym, unsigned char r, unsigned char color1, unsigned char color2)
{	
	unsigned short x;
	unsigned char y=r;
	signed short p;
	if ((r>xm)||(r>ym)||(r<1))
		return;
	y=r;
	p=(3-(2*r));
	for (x=0; x<=y; x++)
	{	lcd_line(xm-x,ym+y,xm+x,ym+y,color1);
		lcd_line(xm-x,ym-y,xm+x,ym-y,color1);
		lcd_line(xm-y,ym+x,xm+y,ym+x,color1);
		lcd_line(xm-y,ym-x,xm+y,ym-x,color1);
		if(p<0)
			p+=((4*x)+6);
		else
		{	y--;
			p+=((4*(x-y))+10);
		}
	}
	y=r;
	p=(3-(2*r));
	for (x=0; x<=y; x++)
	{	lcd_setpixel((xm+x),(ym+y),color2);
		lcd_setpixel((xm+y),(ym+x),color2);
		lcd_setpixel((xm+y),(ym-x),color2);
		lcd_setpixel((xm+x),(ym-y),color2);
		lcd_setpixel((xm-x),(ym-y),color2);
		lcd_setpixel((xm-y),(ym-x),color2);
		lcd_setpixel((xm-y),(ym+x),color2);
		lcd_setpixel((xm-x),(ym+y),color2);
		if(p<0)
			p+=((4*x)+6);
		else
		{	y--;
			p+=((4*(x-y))+10);
		}
	}

}

void lcd_block (unsigned short x1, unsigned short y1, unsigned short x2, unsigned short y2, unsigned char color1, unsigned char color2)
{	
	unsigned short y;
	if (x2>x1+1)
	{	for (y=y1+1; y<y2; y++)
			lcd_line(x1+1,y,x2-1,y,color1);
	}
	lcd_line(x1,y1,x2,y1,color2);
	lcd_line(x2,y1,x2,y2,color2);
	lcd_line(x2,y2,x1,y2,color2);
	lcd_line(x1,y2,x1,y1,color2);
	
}

void lcd_invblock (unsigned short x1, unsigned short y1, unsigned short x2, unsigned short y2)
{	
	unsigned short y,temp;
	
	if (x1>x2)
	{	
		temp=x1;
		x1=x2;
		x2=temp;
	}
	for (y=y1; y<=y2; y++)
	{
		for (temp=x1; temp<=x2; temp++)
		{
			if ((temp & 3)==0 && (x2-temp)>3)
			{
				lcd_invnibbel(temp/4,y);
				temp += 3;
			}
			else
				lcd_invpixel (temp,y);
		}
	}		
	
}
